import logging
import config

try:
    import RPi.GPIO as GPIO
except ImportError:
    GPIO = None


class GPIOInterface:

    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.enabled = False
        if config.get_settings().get('gpio').get('enabled'):
            self.init()

    def init(self):
        self.enabled = False
        if not GPIO:
            self.logger.warning("No RPi.GPIO module - disabling GPIO interface")
            return
        self.logger.info("Enabling GPIO interface")
        trigger_str = config.get_settings().get('gpio').get('trigger')
        if trigger_str == "rising":
            self.trigger = GPIO.RISING
        elif trigger_str == "falling":
            self.trigger = GPIO.FALLING
        elif trigger_str == "both":
            self.trigger = GPIO.BOTH
        else:
            self.logger.warning("Error starting RPi.GPIO: unknown trigger type in config. Should be one of rising, falling, both")
            self.trigger = None
        if not self.trigger:
            self.logger.info("GPIO errir: bad trigger")
        pull_str = config.get_settings().get('gpio').get('pull')
        if pull_str == "up":
            self.pull = GPIO.PUD_UP
        elif pull_str == "down":
            self.pull = GPIO.PUD_DOWN
        else:
            self.pull = None
        self.bouncetime = config.get_settings().get('gpio').get('bouncetime', 200)
        GPIO.setmode(GPIO.BOARD)
        GPIO.setwarnings(False)
        self.active_buttons = []
        self.cancel_callbacks = []
        self.buttons = []
        buttons = config.get_settings()['gpio'].get('buttons', {})
        for button in buttons:
            action_name = button
            pin_number = buttons[button]
            self.buttons.append((pin_number, action_name))
            try:
                if self.pull:
                    GPIO.setup(pin_number, GPIO.IN, pull_up_down=self.pull)
                else: #TODO recheck pull_up_down=None here, should it be so when pull_str is "down"?
                    GPIO.setup(pin_number, GPIO.IN)
                GPIO.add_event_detect(pin_number, self.trigger, bouncetime=self.bouncetime)
            except RuntimeError:
                try:
                    GPIO.cleanup()
                except RuntimeError:
                    pass
                self.logger.exception("Exception during GPIO pins setup:")
                return
            else:
                self.logger.info("GPIO event registered: pin%d=%s" % (pin_number, action_name))
        self.enabled = True
        self.logger.info("GPIO interface init success")

    def check_events(self, printer_interfaces):
        if GPIO and self.enabled:
            for pin_num, action_name in self.buttons:
                try:
                    got_event = GPIO.event_detected(pin_num)
                except RuntimeError:
                    self.logger.exception("GPIO error on adding event to pin " + str(pin_num))
                    self.close()
                    return
                else:
                    if got_event:
                        for pi in printer_interfaces:
                            self.run_method(pi, action_name)

    def run_method(self, pi, method_name):
        method = getattr(pi, method_name, None)
        if method:
            try:
                method()
            except Exception:
                self.logger.exception("Exception in GPIO handler")
        else:
            self.logger.error("GPIO event error: no method %s" % method_name)

    def close(self):
        if GPIO:
            GPIO.cleanup()
        self.enabled = False

    def toggle(self):
        if self.enabled:
            self.close()
        else:
            self.init()
        settings = config.get_settings()
        settings['gpio']['enabled'] = self.enabled
        config.Config.instance().save_settings(settings)
